#########################################################################
#  Metatrans - metatranscriptomic sequence analysis pipeline for the    #
#  human gut microbiota.                                                #
#                                                                       #
#  Copyright (C) 2015 Xavier Martínez VHIR - Vall d'Hebron              #
#                                            Institut de Recerca        #
#                                                                       #
#  This file is part of Metatrans.                                      #
#                                                                       #
#  Metatrans is free software: you can redistribute it and/or modify    #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  Metatrans is distributed in the hope that it will be useful,         #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with Metatrans.  If not, see <http://www.gnu.org/licenses/>.   #
#                                                                       #
#  Contact e-mail:                                                      #
#  metatrans-forum@googlegroups.com                                     #
#########################################################################


## DESCRIPTION
 
#Method described by POSIX to set shebang in the best portable way,
#instead of "/usr/bin/env sh". Modifies headers from files with extension
#.source by replacing the tag "<INSTALLSHELLPATH>" with the full path to "sh".
#http://pubs.opengroup.org/onlinepubs/9699919799/utilities/sh.html#tag_20_117_16


## SHELL OPTIONS

set -e  #Exit from script if any statement return a non-true value. 


## FUNCTIONS

##ADD CORRECT SHELL PATH + LINK TO RUN THE PIPELINE
#Searches tag <INSTALLSHELLPATH> within ".source" script files and
#switch it to the appropiate absolute path of the shell.


translateShell() {

    #Get list of paths to check for 'sh' binary
    Sifs=$IFS
    Sifs_set=${IFS+y}
    IFS=:
    set -- $(getconf PATH)  #This sets $@ with the content of "getconf PATH"
    if [ "$Sifs_set" = y ]
    then
        IFS=$Sifs
    else
        unset IFS
    fi


    #Check each path for 'sh'
    #Reads internally from $@
    for i   
    do
        if [ -x "${i}"/sh ]
        then
        Pshell=${i}/sh
        fi
    done

    #List of scripts to modify
    PIPELINESCRIPTS='./0-Software/pipelinescripts'
    INHOUSESCRIPTS='./0-Software/scripts'

    scripts="$PIPELINESCRIPTS/pipeline \
    $PIPELINESCRIPTS/run_qc            \
    $PIPELINESCRIPTS/run_rrnaremoval   \
    $PIPELINESCRIPTS/run_map           \
    $PIPELINESCRIPTS/run_map_16S       \
    $INHOUSESCRIPTS/cfqf               \
    $INHOUSESCRIPTS/reservoir_sampling_fasta"



    #Replace all <INSTALLSHELLPATH> tags
    #Extension 'SCRIPTNAME.source' changed to 'SCRIPTNAME.sh'
    tag='<INSTALLSHELLPATH>'
    for i in ${scripts}
    do
        #Remove spaces, tabs, carriage returns and newlines
        h=$(head -n1 ${i}.source | tr -d '\040\011\012\015')  
        #Check if the first line contains "<INSTALLSHELLPATH>"
        if [ "$h" = "${tag}" ]; then
            sed -e "s|${tag}|#!${Pshell}|" < ${i}.source > ${i}.sh  #Change tag
            chmod ug+x ${i}.sh
            rm -f ${i}.source
        else 
            printf "#Error# The source shell file \"${i}.source\" did not have \"${tag}\" tag.\n"
            exit 1
        fi
    done
}


## MAIN CODE

printf "#Configuring MetaTrans#\n";

printf " #Translating shell paths ...\n";
translateShell

#Ensure permissions are set correctly
#Config files
printf " #Setting config file permissions ...\n";
find ./0-Software/config -type f -print0 | xargs -0 chmod 644
find ./0-Software/config -type d -print0 | xargs -0 chmod 755

#Manual
printf " #Setting user manual file permissions ...\n";
set +e  #Disable exit from script if returns a non-true value
find . -name 'USER_MANUAL.pdf' -type f -print0 | xargs -0 chmod -f 644   2>/dev/null  1>&2
set -e

#Pipeline scripts
printf " #Setting pipeline scripts permissions ...\n";
find ./0-Software/pipelinescripts -type f -print0 | xargs -0 chmod 755
find ./0-Software/pipelinescripts -type d -print0 | xargs -0 chmod 755

#Auxiliar scripts
printf " #Setting auxiliar scripts permissions ...\n";
find ./0-Software/scripts -type f -print0 | xargs -0 chmod 755
find ./0-Software/scripts -type d -print0 | xargs -0 chmod 755

#Thirdparty tools
printf " #Setting thirdparty tools permissions ...\n";
find ./0-Software/thirdpartytools -type d -print0 | xargs -0 chmod 755   #Folders must be accessible
find ./0-Software/thirdpartytools -type f -exec sh -c "file -i '{}' | grep -q 'x-executable; charset=binary'" \; -print0 |xargs -0 chmod 755                                       #Binary files
find ./0-Software/thirdpartytools \( -name '*.sh' -or -name '*.pl' -or -name '*.py' -or -name '*.R' -or -name '*.awk' \) -type f -print0 | xargs -0 chmod 755                      #Common shellscript extensions
find ./0-Software/thirdpartytools -type f -exec grep -Iq . {} \; -and -print0 | xargs -0 grep -nH '#!'|grep ':1'|cut -d ':' -f1|grep -v '~$'  |tr "\n" "\0" | xargs -0 chmod 755   #Shellscripts without common extensions or none

#Create a symlink to main script and configuration files
printf " #Creating symlinks ...\n";
ln -f -s ./0-Software/pipelinescripts/pipeline.sh ./metatrans
ln -f -s ./0-Software/config/metadata.txt         ./metadata.txt
ln -f -s ./0-Software/config/settings.txt         ./settings.txt

printf "#Initial configuration done.\n";



exit 0


