#########################################################################
#  Metatrans - metatranscriptomic sequence analysis pipeline for the    #
#  human gut microbiota.                                                #
#                                                                       #
#  Copyright (C) 2015 Xavier Martínez VHIR - Vall d'Hebron              #
#                                            Institut de Recerca        #
#                                                                       #
#  This file is part of Metatrans.                                      #
#                                                                       #
#  Metatrans is free software: you can redistribute it and/or modify    #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  Metatrans is distributed in the hope that it will be useful,         #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with Metatrans.  If not, see <http://www.gnu.org/licenses/>.   #
#                                                                       #
#  Contact e-mail:                                                      #
#  metatrans-forum@googlegroups.com                                     #
#########################################################################

## COMMON CODE AND FUNCTIONS TO ALL SCRIPTS. ALL THEM IMPORT THIS FILE.

## SHELL ENVIORNMENT VARIABLES

export PID=$$          #Get current PID


## SHELL OPTIONS

set -u #Enables uninitialised variables checking
set -e #Exit from script if any statement return a non-true value. 
       #Unfortunately it means you can’t check $? as sh will never get to the 
       #checking code if it isn’t zero.

#Save shell options status
export SHSTATE="$(set +o)"

## ALIAS (must be placed in a sourced file, since cannot be exported)

alias rm='rm -i' #Avoid accidental use of rm within the pipeline if "-f" flag is 
                 #not set.

##FUNCTIONS

#Set time output and format.
#An additional substring can be added to modify the timing output filename.
#Format: set_timef [<additional_text>]
#E.g: set_timef '-16S' , output filename (if we are in module3): $DLOG/m3-16S-time.tsv
#1) C           Name and command line arguments of the command being timed.
#2) U (user)    Total number of CPU-seconds that the process used directly (in user mode), in seconds.
#3) S (sys)     Total number of CPU-seconds used by the system on behalf of the process (in kernel mode), in seconds.
#4) e (real)    Elapsed real (wall clock) time used by the process, in seconds.
#5) P           Percentage of the CPU that this job got.  This is just user + system times divided by the total running time.  It also
#               prints a percentage sign.
#6) M           Maximum resident set size of the process during its lifetime, in Kilobytes.
#7) I           Number of file system inputs by the process.
#8) O           Number of file system outputs by the process.
set_timef() {
    
    set +u    #Avoid checking if $1 is empty
    
    #Print time header
    printf "Command\tUser\tSys\tReal(wall_clock)\tCPU%%\tMaxResident(Kb)\tInputs\tOutputs\n" >> "$DLOG/m${module}${1}-time.tsv"
    
    #Variable expansion of this alias is performed when this function is called within the modules.
    alias time="/usr/bin/time --format='\"%C\"\t%U\t%S\t%e\t%P\t%M\t%I\t%O' -a -o $DLOG/m${module}${1}-time.tsv "
    
    eval "$SHSTATE"   #Restore shell options status
}

#Print out a text passed as argument if environment is in DEBUG mode.
debug() {

    if [ "$DEBUG" = true ]; then
        printf "$1\n" 
    fi
}


#Simple commmand line calculator
#Eg.: calc 3+2 ; calc 3/2 ; calc 3/2. (with float division rounded to 1 decimal)
calc() { 
    awk "BEGIN { 
        input=\""$*"\"; 
        if(substr(input,length(input),1) == \".\") {
            text=sprintf( \"%3.1f\", "$*" ); 
            sub(/\.*0+$/,\"\",text); print text;
        } else { print int("$*"); }  
    }"; 
}  

#Split string argument "$1" using separator(s) in string argument "$2" and
#assign split substrings to global variables given as arguments $3..$<n>.
#Eg.:
#       split 'first item,second:third' ',:' one two three
#       string='first,second'
#       split "$string" ',' f1 f2
#       echo $f1; echo $f2
#
#will split string 'first item,second:third' using ',' and ':' as separators
#and assign substrings to variables:
#       'first item' to global variable "one",
#       'second'     to global variable "two", and
#       'third'      to global variable "three".
#Allows filenames with whitespaces.
#Warning: Don't use local variable names ("_s1", "_g1", "IFS") for the
#       global variables passed (arguments $3..$<n>). 
#Original author: Julien Fontanet , Wolfgang Reimer <w.r (at) gmx.org>
split() {
    local _s1 IFS _g1
    _s1=$(printf "$1\n"|tr ' ' "'")   #Delimiter ' used as very improbable to be in a filename 
    IFS="$2"
    shift 2
    read -r -- "$@" _g1 <<EOF
$(printf "$_s1\n"|tr "'" ' ')
EOF
}

printDate() {
    printf "$(date +"Date: '%d-%m-%y' Time: '%Hh:%Mm:%Ss %Z'")\n"
    #Eg.: Date: '27-05-15' Time: '23h:28m:50s CEST'
}

#Modify a shell variable stored in a file.
#Preserve single or double quotes of the var modified if it has.
#Format: store_var <key> <value> <file>
#E.g:  
#$ cat .foofile
#MAPFOLDER='whatever'
#store_var MAPFOLDER '/foldername' .foofile
#$ cat .foofile
#MAPFOLDER='/foldername'
store_var() {  
VAL=$(echo "$2" | sed 's#\\#\\\\#g')
awk -v key="$1" -v value="$VAL"   '
    function getfieldtillend(nfield) 
    {
        out="";
        for (i = nfield; i <= NF; i++) { 
            out = out $i " ";
        }                             
        return out;
    }
    $0 ~ "^export[[:space:]]"key"="  {
        split($0,a,"=");
                if(substr(a[2],1,1) == "'\''") {
            split(substr(a[2],2),b,"'\''");
            print a[1] "='\''" value "'\''" b[2];
        } else if(substr(a[2],1,1) == "\""){
            split(substr(a[2],2),b,"\"");
            print a[1] "=\"" value "\"" b[2];
        } else {
            print a[1] "=" value getfieldtillend(2);
        }
    }
    $0 !~ "^export[[:space:]]"key"=" {print $0} ' "$3" > tmp && mv -f tmp "$3"
}

#Save those global variables required for the pipeline
#to continue the job if the pipeline is partially run due
#to user intervention or crash.
#Pipeline control file: ./0-Software/config/.pipeline_control
save_status() {

    #Check if control file exist
    FCONTROL='./0-Software/config/.pipeline_control'
    if [ ! -f "$FCONTROL" ]; then 
        printf "\n #Error# Pipeline control file not found. \n" >&2 
        printf "\n Unable to save pipeline status. \n" >&2
        exit 1; 
    fi

    #Check variables are set and have a value
    
    set +u          #If enabled, disable auto-checking for unset variables
    
    if [  ! -z ${MAPFOLDER+x} ] || [  ! -z ${MAPOUTPUTFOLDER+x} ] \
       || [  ! -z ${MAP16SFOLDER+x} ] || [  ! -z ${MAP16SOUTPUTFOLDER+x} ]; then

       if env | grep -q "^MAPFOLDER=" &&  env | grep -q "^MAPOUTPUTFOLDER="  \
          && env | grep -q "^MAP16SFOLDER=" && env | grep -q "^MAP16SOUTPUTFOLDER="; then
                if [ "$MAPFOLDER" = '' ] || [ "$MAPOUTPUTFOLDER" = '' ] \
                    || [ "$MAP16SFOLDER" = '' ] || [ "$MAP16SOUTPUTFOLDER" = '' ]; then
                    printf "#WARNING# There are pipeline control variables set to empty strings in '.pipeline_control'.\n" >&2 
                fi
       else
           printf "\n #Error# There are pipeline control variables unexported in '.pipeline_control'.\n " >&2 ; exit 1;
       fi
    else
        printf "\n #Error# There are pipeline control variables unset in '.pipeline_control'.\n " >&2 ; exit 1;
    fi

    store_var "MAPFOLDER" "$MAPFOLDER" "$FCONTROL"
    store_var "MAPOUTPUTFOLDER" "$MAPOUTPUTFOLDER" "$FCONTROL"
    store_var "MAP16SFOLDER" "$MAP16SFOLDER" "$FCONTROL"
    store_var "MAP16SOUTPUTFOLDER" "$MAP16SOUTPUTFOLDER" "$FCONTROL"
    
    eval "$SHSTATE"   #Restore shell options status
}


## ERROR HANDLING

#Handle/trap pipeline aborts.
# INT(2): for keyboard interrupt (Ctrl+C)
# TERM(15): clean termination of a process (kill <PID>). Politely ask a process to terminate. It shall terminate gracefully, cleaning up 
#           all resources (files, sockets, child processes, etc.), deleting temporary files and so on.
# EXIT: exit command.  If the script exits on error your EXIT trap is executed. (when finishing a script properly always executes: exit 0)
killed_by=''    
trap killed_by=Ctrl+C INT
trap killed_by=Termination TERM
trap '
ret=$? 

#Remove old pipe and create a new pipe to append "trap-code" block errors to "$ferr"
rm -f "$fifo_err"  1>/dev/null 2>&1
err="${TMPDIR:-/tmp}/err.$$.$(date +"%H_%M_%S")"
mkfifo "$fifo_err"
tee -a "$ferr" < "$fifo_err" >&2 &

stderr_pid2=$!
  {
  set +u          #Disable auto-checking for unset variables
  set +e          #Disable exit from script if any statement return a non-true value
  if [ -n "$killed_by" ] && [  $PID -eq $PAPID ]; then
            #PARENT (kill and Ctrl+C are controlled by the parent)
            if [ "$killed_by" = "Ctrl+C" ]; then
               printf "\n#Pipeline aborted by the user(Ctrl+C). \n" 1>&2
            elif [ "$killed_by" = "Termination" ]; then
               #The process was killed
               printf "\n #Error# Pipeline terminated unexpectedly. \n" 1>&2
            fi
            save_status
            kill -s 0 $stderr_pid2 1>/dev/null 2>&1 && kill -15 $stderr_pid2 1>/dev/null 2>&1
            rm -f "$fifo_out" "$fifo_err"  1>/dev/null 2>&1
            exit 1
        
  elif [ "$ret" -ne 0 ]; then
        if [  $PID -ne $PAPID ]; then
            #CHILD
             exit $ret
        else
            #PARENT
            printf "\n #Error# Pipeline exited with error code $ret. \n" 1>&2
            printf "\n #HELP# An error may lead files in an intermediate state. To re-run the pipeline some folder might be deleted. \n" 1>&2
            printf " #HELP# Please, read the troubleshooting section of the manual for help. \n" 1>&2
            save_status
            kill -s 0 $stderr_pid2 1>/dev/null 2>&1 && kill -15 $stderr_pid2 1>/dev/null 2>&1
            rm -f "$fifo_out" "$fifo_err"  1>/dev/null 2>&1
            exit $ret
        fi
  elif [ "$ret" -eq 0 ]; then
        kill -s 0 $stderr_pid2 1>/dev/null 2>&1 && kill -15 $stderr_pid2 1>/dev/null 2>&1
        rm -f "$fifo_out" "$fifo_err"  1>/dev/null 2>&1
        exit 0
    fi
  } 2>"$fifo_err"  1>&2
' EXIT
  
