###DESeq2###

#########################################################################
#  Metatrans - metatranscriptomic sequence analysis pipeline for the    #
#  human gut microbiota.                                                #
#                                                                       #
#  Copyright (C) 2015 Xavier Martínez VHIR - Vall d'Hebron              #
#                                            Institut de Recerca        #
#                                                                       #
#  This file is part of Metatrans.                                      #
#                                                                       #
#  Metatrans is free software: you can redistribute it and/or modify    #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  Metatrans is distributed in the hope that it will be useful,         #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with Metatrans.  If not, see <http://www.gnu.org/licenses/>.   #
#                                                                       #
#  Contact e-mail:                                                      #
#  metatrans-forum@googlegroups.com                                     #
#########################################################################

## Description

# Differential expression analysis based on the negative binomial distribution (DESeq2)
# This function performs a default analysis by calling, in order, the functions:
#   estimateSizeFactors, 
#   estimateDispersions, 
#   nbinomWaldTest.

# betaPrior  : whether or not to put a zero-mean normal prior on the non-intercept coefficients (Tikhonov/ridge regularization)
#             See nbinomWaldTest for description. Only used for the Wald test.


## Parse arguments

args <- R.utils::commandArgs(asValues=TRUE)

if(is.null(args$fin)) {
   stop("--fin not set")
} else { 
   fin <- args$fin 
}

if(is.null(args$dout)) {
   stop("--dout not set")
} else {
   dout <- args$dout   
}

if(is.null(args$fout)) {
   stop("--fout not set")
} else {
   fout <- args$fout
}

if(is.null(args$fdr)) {
   stop("--fdr not set")
} else {
   fdr <- as.numeric(args$fdr)
}

if(is.null(args$metadata)) {
   stop("--metadata file not specified")
} else {
   metadata <- args$metadata 
}

if(!is.null(args$debugging)){
  if(args$debugging == "true") { 
    debugging <- TRUE
  } else { debugging <- FALSE }
}

if(debugging){write("## Debugging R code",stderr())}



## Load library DESeq2
library("DESeq2")


## Make outupt folder
dir.create(dout, showWarnings = TRUE, recursive = TRUE, mode = "0777")
setwd(dout)


## Set output file handler
fileConn<-file(paste(dout,fout,".deseq2summary.txt",sep=''),"w") #Write data to summary file. If we don't specify: "w" , all writeLines functions overwrite same line (don't append)


## Read input
CountTable <- read.table(fin, header=TRUE, row.names=1,sep="\t")

## Read metadata
MetaTable<-read.table(metadata, header=TRUE, sep="\t",comment.char = ";")
colnames(MetaTable)[1]<-sub('X.','',colnames(MetaTable)[1])


## Get list of columns that have factors. 
#"read.table" convert "-" to "."
#And create a mapping bw column number and factor number.
fcolnums<-as.integer(grep("F[0-9]+\\.",colnames(MetaTable), value=FALSE)) #return cols where there are factors to parse
fcolnames<-grep("F[0-9]+\\.",colnames(MetaTable), value=TRUE)  #"\\." to avoid interpret dot as any character in a regex
fnums<-as.integer(regmatches(fcolnames,regexpr("[0-9]+",fcolnames))) #return factors order set by the user
fnumTofcol<-rbind(fcolnums,fnums)
colnames(fnumTofcol)<-fcolnames
fnumTofcol<-fnumTofcol[,order(fnumTofcol[2,])]  #Order by "fnums" (2nd row)

## Remove all factors with "F0"
fnumTofcol<-subset(fnumTofcol,select=grep("0+",colnames(fnumTofcol),invert = TRUE)) #Subselect columns using a vector of integers (column numbers)

## Ensure Order column are integers
MetaTable$Order<-as.integer(MetaTable$Order) 

## Create a dataframe with shortname sample ids and all factors
CountData<-data.frame(Shortname=as.factor(MetaTable[MetaTable$Order>0,]$Shortname)) #Add sample shortname ids

for( j in 1:dim(fnumTofcol)[2] ) {
    fcol=fnumTofcol[1,j]  #Get the column of the table that corresponds to that factor
    CountData<-cbind.data.frame( CountData,as.factor(MetaTable[MetaTable$Order>0,][, fcol ]) )
    colnames(CountData)[dim(CountData)[2]]<-colnames(MetaTable)[fcol]  #Set colname
}

## Re-assign levels to factors
CountData <- as.data.frame(
    lapply(CountData, 
           function(x) if(is.factor(x)) droplevels(x) else x
    )
)


#Note DESeq2: In order to benefit from the default settings of the package, you should put the variable of interest
#at the end of the formula and make sure the control level is the first level.

## Set control level as first level
lastFactor<-dim(CountData)[2]
baseline<-as.character(unique(CountData[,lastFactor])[1])  #Get the left-most (1st) level, which is considered to be the baseline
CountData[,lastFactor] <- relevel(CountData[,lastFactor],baseline) #Set the control level (baseline) as the 1st level. REQUIRED.

## Assign sampleids (1st CountData column) as rownames and remove the column.
rownames(CountData) <- CountData[,1]
CountData <- CountData[,-1,drop=FALSE]  #"drop=FALSE" avoid auto. conversion from dataframe->vector, when only 1 column is left

## Set design formula (variable(factor) of interest must be the last):
listoffactors <- colnames(CountData)
CountTable <- CountTable[,rownames(CountData)]
colnames(CountTable) <- NULL 
textdata<-paste0("Design Formula used (variable of interest at the end of the formula): ",paste("~",paste(listoffactors,collapse="+")))
print(textdata)
writeLines(textdata, fileConn)
textdata<-paste0("Last Factor (factor of interest) is: '",colnames(CountData)[lastFactor-1],"', Control level for that factor is: '",baseline,"'")
print(textdata)
writeLines(textdata, fileConn)

ddsMF <- DESeqDataSetFromMatrix(countData = CountTable,
                                colData = CountData,
                                design =  as.formula(paste("~",paste(listoffactors,collapse="+"))))

ddsMF <- DESeq(ddsMF)


## Write out normalized table.
write.table(counts(ddsMF,normalized=TRUE),
            file=paste(dout,fout,".deseq2normalised.tsv",sep=''),sep = "\t",col.names=NA)

resMF <- results(ddsMF)

## To add SD we need to supply equivalent rowMeans/colMeans functions for standard deviation:
colSdColMeans <- function(x, na.rm=TRUE) {
  if (na.rm) {
    n <- colSums(!is.na(x)) 
  } else {
    n <- nrow(x)
  }
  colVar <- colMeans(x*x, na.rm=na.rm) - (colMeans(x, na.rm=na.rm))^2
  return(sqrt(colVar * n/(n-1)))
}

rowSdRowMeans <- function(x, na.rm=TRUE) {
  if (na.rm) {
    n <- rowSums(!is.na(x))
  } else {
    n <- ncol(x)
  }
  rowVar <- rowMeans(x*x, na.rm=na.rm) - (rowMeans(x, na.rm=na.rm))^2
  return(sqrt(rowVar * n/(n-1)))
}


lastFactor<-dim(CountData)[2]
resMFexp<-matrix()
tempSD<-matrix()
LF<-colnames(CountData)[lastFactor]  #Last factor name. To access by colname using a "string", need to
                                     # do: ddsMF[[LF]] instead of ddsMF$LF .
                                     #  ddsMF[[1]] : 1st factor, ddsMF[[2]] : 2nd factor, ...
LFL<-levels(CountData[,lastFactor])  #Last factor levels
for( i in 1:length(LFL) ){
    if(i == 1){
        resMFexp<-as.matrix( rowMeans( counts(ddsMF,normalized=TRUE)[ ,ddsMF[[LF]] == LFL[i] ] ) )
        colnames(resMFexp)[i]<-paste("baseMean.",LFL[i], sep = "")
        tempSD<-as.matrix( rowSdRowMeans( counts(ddsMF,normalized=TRUE)[ ,ddsMF[[LF]] == LFL[i] ] ) )
        colnames(tempSD)[i]<-paste("SD.",LFL[i], sep = "")
    } else {
        resMFexp<-cbind( resMFexp, as.matrix( rowMeans( counts(ddsMF,normalized=TRUE)[ ,ddsMF[[LF]] == LFL[i] ] ) ) )
        colnames(resMFexp)[i]<-paste("baseMean.",LFL[i], sep = "")
        tempSD<-cbind( tempSD, as.matrix( rowSdRowMeans( counts(ddsMF,normalized=TRUE)[ ,ddsMF[[LF]] == LFL[i] ] ) ) )
        colnames(tempSD)[i]<-paste("SD.",LFL[i], sep = "")
    }
}
resMFexp <- cbind(resMFexp,tempSD,as.data.frame(resMF)) #This results (that include extra info) are called "resMFExpanded"
remove(tempSD)

## We can order our results table by the smallest (FDR) adjusted p value:  
resOrdered <- resMFexp[order(resMFexp$padj),] #From smallest FDR to the highest

#Count how many DE (FDR<0.1) genes we have:
#View(resOrdered)
#This variable "fdr.thresold" is set on top of this script.
#fdr.threshold=0.1 #FDR <10%
#fdr.threshold=0.05 #FDR <5%
textdata<-paste0(sum( resMFexp$padj < fdr, na.rm=TRUE )," functions DE, out of ",nrow(resOrdered))
print(textdata)
writeLines(textdata, fileConn)
#In percentage, the number of DE genes are:
textdata<-paste0(round((sum( resMFexp$padj < fdr, na.rm=TRUE )/nrow(resOrdered))*100, digits=2),"% of functions are DE")
print(textdata)
writeLines(textdata, fileConn)


## DE genes call (example: https://support.bioconductor.org/p/58337/):

# Check all genes with FDR<0.1:
# We subset the results table to these genes and then sort it by the log2-fold-change estimate to get
# the significant genes with the strongest down-regulation
resSig <- subset(resMFexp, resMFexp$padj < fdr )
log2FCAsc<-resSig[ order( resSig$log2FoldChange ),]
resSigDOWN <- subset(log2FCAsc, log2FCAsc$log2FoldChange < 0 ) 
write.table(as.data.frame(resSigDOWN),
            file=paste(dout,fout,"-DOWNREGULATED-FDR-",fdr,".tsv",sep=''),sep = "\t",col.names=NA) #Using sep = "\t" gives tab delimited output; default is " "

log2FCDesc<-resSig[ order( -resSig$log2FoldChange ),]
resSigUP <- subset(log2FCDesc, log2FCDesc$log2FoldChange > 0 )
write.table(as.data.frame(resSigUP),
            file=paste(dout,fout,"-UPREGULATED-FDR-",fdr,".tsv",sep=''),sep = "\t",col.names=NA) #Using sep = "\t" gives tab delimited output; default is " "


## Now we visualize that data graphically with "plotMA":
png(paste(dout,fout,"-plotMA.png",sep=''),width = 480, height = 480, units="px",res=80)
plotMA(ddsMF,ylim=c(-2,2),main="DESeq2")
dev.off()


## Exploring counts table:
#2.2 - Data quality assessment by sample clustering and visualization
#2.2.1 - Heatmap of the count table
library("RColorBrewer")
library("gplots")

# Heatmaps showing the expression data of the N most highly expressed genes.
N=10  #N most highly expressed genes.
select <- order(rowMeans(counts(ddsMF,normalized=TRUE)),decreasing=TRUE)[1:N]
hmcol <- colorRampPalette(brewer.pal(9, "GnBu"))(100)

# Regularized log transformation
rld <- rlogTransformation(ddsMF, blind=TRUE)

png(paste(dout,fout,"-LogTransf-HeatmapCounts_",N,"genes_most_expressed.png",sep=''),width = 800, height = 600, units="px",res=80)
heatmap.2(assay(rld)[select,], col = hmcol,
          Rowv = FALSE, Colv = FALSE, scale="none",
          dendrogram="none", trace="none", margin=c(10, 24))
dev.off()



# 2.2.2 - Heatmap of the sample-to-sample distances
# Another use of the transformed data is sample clustering. Here, we apply the dist function to the
# transpose of the transformed count matrix to get sample-to-sample distances.
distsRL <- dist(t(assay(rld)))
mat <- as.matrix(distsRL)
for( i in 1:dim(CountData)[2]){
    if(i == 1){
        llabels<-ddsMF[[i]]
    } else {
        llabels<-paste(llabels,ddsMF[[i]],sep=" : ")
    }
}
rownames(mat) <- colnames(mat) <- llabels

png(paste(dout,fout,"-Heatmap_Sample_to_Sample_distances.png",sep=''),width = 480, height = 480, units="px",res=80)
heatmap.2(mat, trace="none", col = rev(hmcol), margin=c(13, 13))
dev.off()


# 2.2.3 - Principal component plot of the samples
library("ggplot2")


data <- plotPCA(rld, intgroup = c( colnames(CountData)[1], colnames(CountData)[dim(CountData)[2]]), returnData=TRUE)

percentVar <- round ( 100 * attr (data, "percentVar" )) 

intgroup <- colnames(CountData)[1:dim(CountData)[2]-1]
intgroup.df <- as.data.frame(colData(rld)[, intgroup, drop = FALSE])
group <- factor(apply(intgroup.df, 1, paste, collapse = " : "))
data$group <- group   #Overwrite "group" column autogenerated by "plotPCA" with our customized grouping


png(paste(dout,fout,"-PCA.png",sep=''),width = 670, height = 480, units="px",res=80)
if(length(intgroup) == 0) { #Last factor is the factor of interest, the others are collapsed to one for the legend representation
    ggplot (data, aes_string("PC1", "PC2", shape = colnames(CountData)[dim(CountData)[2]] , colour = colnames(CountData)[dim(CountData)[2]] ) ) + 
      geom_point ( size = 7 ) +
      xlab ( paste0 ( "PC1: " ,percentVar[ 1 ], "% variance" )) +
      ylab ( paste0 ( "PC2: " ,percentVar[ 2 ], "% variance" )) +
     theme(  text = element_text(size=24), axis.text.x = element_text(vjust=1)) 
} else {
    ggplot (data, aes_string("PC1", "PC2", color='group', shape = colnames(CountData)[dim(CountData)[2]] ) ) + 
      scale_colour_discrete(name=paste(intgroup,collapse=" : "),labels=group) +
      scale_shape_discrete(name=colnames(CountData)[dim(CountData)[2]]) +
      guides(colour = guide_legend(order = 1), 
               shape = guide_legend(order = 2)) +
      geom_point ( size = 7 ) +
      xlab ( paste0 ( "PC1: " ,percentVar[ 1 ], "% variance" )) +
      ylab ( paste0 ( "PC2: " ,percentVar[ 2 ], "% variance" )) +
     theme(  text = element_text(size=24), axis.text.x = element_text(vjust=1))
}


dev.off()


#In ".tsv" format:
write.table(as.data.frame(resMFexp),
            file=paste(dout,fout,".deseq2results.tsv",sep=''),sep = "\t",col.names=NA) #Using sep = "\t" gives tab delimited output; default is " "

close(fileConn)   #Close summary file.

