#!/usr/bin/env python
# -*- coding: UTF-8 -*-


#########################################################################
#  Metatrans - metatranscriptomic sequence analysis pipeline for the    #
#  human gut microbiota.                                                #
#                                                                       #
#  Copyright (C) 2015 Xavier Martínez VHIR - Vall d'Hebron              #
#                                            Institut de Recerca        #
#                                                                       #
#  This file is part of Metatrans.                                      #
#                                                                       #
#  Metatrans is free software: you can redistribute it and/or modify    #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  Metatrans is distributed in the hope that it will be useful,         #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with Metatrans.  If not, see <http://www.gnu.org/licenses/>.   #
#                                                                       #
#  Contact e-mail:                                                      #
#  metatrans-forum@googlegroups.com                                     #
#########################################################################


"""
Description:
    Annotates all M5nr md5 ids from a Diamond tabular output file,
    to EggNOG or COG orthologous ids.

    Input,e.g:
         141633_t9_w76_x5_1_113_-_xcs17     c914fc2d627ac751e755acd04775d9f5  81.2   32    5     1     1     93    70    101    5.1e-04  50.1
         161179_t10_w76_x5_1_144_+_xcs168   9f1039c5f6707e58139bb66648eb2f20  82.9   41    7     0     1     123   31    71     1.7e-09  68.6

    Output,e.g:
         79465  unknown
         8075   NOG12793
         2506   COG1653
         2473   COG0574
         2267   COG0642
         2146   NOG298297

    Usage:  
        DIAMOND_m5nrToOrthids.py [--help | --dicin1=<filename> --fin=<filename>]

        or, in short form,

        DIAMOND_m5nrToOrthids.py [-h | -d1 <filename> -i <filename>]

        E.g:
            DIAMOND_m5nrToOrthids.py -d1 ./0-Databases/annotations_db/eggNOG.md52id2ont -i ./3-MAP/m3output/diamond/C1JB3ACXX_1_3_m3.m5nr.m8.tsv  > out.txt
"""

from __future__ import print_function
from sys import stderr


def mesgstderr(*objs):
    print("", *objs, file=stderr)


## Define Imports
import warnings
warnings.filterwarnings('ignore', '.*the popen2 module is deprecated.*', DeprecationWarning)


import time
from time import ctime #to define ctime()
start_time = time.time()

from sys      import argv, exit, stdout


## Define functions
def dictinit():

    nr=False
    ncs=False
    nfc=False


    argidx = 1    # skip arg 0
    while argidx < len(argv):
        if   argv[argidx] == '-h' or argv[argidx] == '--help':
            usage()
            exit(-1)
        elif argv[argidx] == '-nr':   #Don't count "collapsing" data from headers (e.g.: "..x45")   
                                      #+ we spread out abundance according to the Multi-Letter funcat code
            nr=True
            argidx += 1
        elif argv[argidx] == '-ncs':  #Don't count "cluster size" data from headers (e.g.: "..xcs23")
            ncs=True
            argidx += 1
        elif argv[argidx] == '-nfc':  #No function collision (functions without collision): We only count the 1st function for one read   
            nfc=True
            argidx += 1
        elif argv[argidx] == '-d1':    
            dic1 = argv[argidx+1]
            argidx += 2
        elif argv[argidx].startswith('--dicin1'):
            splitArg = argv[argidx].split('=')
            try:
                dic1 = splitArg[1]
            except:
                usage()
                print('\n #Error# Invalid argument:', argv[argidx],' \n')
                exit(-1)
            argidx += 1
        elif argv[argidx] == '-i': 
            fin = argv[argidx+1]
            argidx += 2
        elif argv[argidx].startswith('--fin'):
            splitArg = argv[argidx].split('=')
            try:
                fin = splitArg[1]
            except:
                usage()
                print('\n #Error# Invalid argument:', argv[argidx],' \n')
                exit(-1)
            argidx += 1
        else:
            usage()
            print('\n #Error# Unrecognized argument:', argv[argidx],' \n')
            exit(-1)


    fileDic = dic1
    #./0-Databases/annotations_db/eggNOG.md52id2ont
    
    #E.g. datafile entry example:
    #000006c1c3a2689dd37477e976a239a1        NOG12793        Calcium ion binding protein     NOG
    #00000de6b0798452314fd6e2af8e8652        NOG70281        Putative uncharacterized protein        NOG
    #00001508eba3f78863a4f9cb2463810d        COG0482         Predicted tRNA(5-methylaminomethyl-2-thiouridylate) methyltransferase, contains the PP-loop ATPase domain       COG
    #c5cc706db0c8126e9100d552bb1ecae8        COG1185         Polyribonucleotide nucleotidyltransferase (polynucleotide phosphorylase)   COG
    #c5cc706db0c8126e9100d552bb1ecae8        NOG239818       Polyribonucleotide nucleotidyltransferase  NOG
    #bbc4607fd48c2ddb7d5c8e1091dc3ffa        NOG05757        Uncharacterized protein    NOG
    #bbc4607fd48c2ddb7d5c8e1091dc3ffa        NOG05757        UPF0403 protein MW1467 NOG
    #bbc4607fd48c2ddb7d5c8e1091dc3ffa        NOG05757        UPF0403 protein SA1345 NOG
    #bbc4607fd48c2ddb7d5c8e1091dc3ffa        NOG05757        UPF0403 protein SAB1387c   NOG
    #bbc4607fd48c2ddb7d5c8e1091dc3ffa        NOG05757        UPF0403 protein SACOL1558  NOG
    #bbc4607fd48c2ddb7d5c8e1091dc3ffa        NOG05757        UPF0403 protein SAOUHSC_01610  NOG
    #bbc4607fd48c2ddb7d5c8e1091dc3ffa        NOG05757        UPF0403 protein SAS1453    NOG
    #bbc4607fd48c2ddb7d5c8e1091dc3ffa        NOG05757        UPF0403 protein SAUSA300_1463  NOG
    #bbc4607fd48c2ddb7d5c8e1091dc3ffa        NOG05757        UPF0403 protein SAV1514    NOG

    #E.g. dictionary entry example (d1cog):
    #c5cc706db0c8126e9100d552bb1ecae8   COG1185;NOG239818
    #bbc4607fd48c2ddb7d5c8e1091dc3ffa   NOG05757
    
    
    ## Load dictionaries to memory for fast access
    d1cog  = {}    #Dictionary of M5nr md5 id to COG ontology ids
    d1nog  = {}    #Dictionary of M5nr md5 id to NOG ontology ids 
    
    #We need to handle collisions for each annotation with same md5id.
    #We keep in a list with ";" all COG/NOG collisions
    with open(fileDic) as f:
        for line in f:
            line=line.strip()
            key=line.split("\t")[0]
            sid=line.split("\t")[3]      #database source id
            oid=str(line.split("\t")[1]) #ontology id
            if sid == "COG": 
                if not d1cog.has_key(str(key)):
                    d1cog[key]=oid
                else:
                    clist=d1cog[key].split(";") #collision list
                    if not oid in clist:
                        d1cog[key]=d1cog[key]+";"+oid  #add new item
            elif sid == "NOG": 
                if not d1nog.has_key(str(key)):
                    d1nog[key]=oid
                else:
                    clist=d1nog[key].split(";") #collision list
                    if not oid in clist:
                        d1nog[key]=d1nog[key]+";"+oid  #add new item

    #print(fileDic) #debug
    
    mesgstderr( "#Finish loading dictionary. It lasted --- %s seconds ---." % (time.time() - start_time) )


    ## Parse input file:
    archivo = open(fin) #Read diamond output. It returns an iterator which reads the file on demand, does  not put the whole file into memory
    
    #E.g.:  
    #141633_t9_w76_x5_1_113_-_xcs17 c914fc2d627ac751e755acd04775d9f5    81.2    32  5   1   1   93  70  101 5.1e-04 50.1
    #141633_t9_w76_x5_1_113_-_xcs17 1fd7e8e74d83b08ef6320a6cda3cfcab    76.9    26  6   0   6   83  70  95  6.4e-04 49.7


    d2={} #Dictionary of orthids and their abundance
    #E.g.: (key,value)="NOG345",73
    #      (key,value)="COG324",765


    #Begin parsing the file
    unknown=0
    for line in archivo:  

        #Parse line (read-hit)
        line = line.strip('\n')  
        if not line: break    
        #print line #debug
        
        md5id=line.split("\t")[1]    #Index starts at 0. This returns the md5 id($2) of the diamond hit
        occField=line.split("\t")[0] #Num of occurrences field (generated at collapsing step)
                
        #If we set "non-redundant" flag, we don't count collapsed copies (artificial duplicate reads) of a read.
        if nr == True:
            occ=1
        else:
            occ=occField.split("_")[3] #E.g.: 254208_t11_w76_x5_1_76_+ , the field num 3 (from 0) contains the counting of occ.
            occ=int(occ.split("x")[1]) #Create an array with 2fields, the 2 parts of the split; in this case occ.split("x")[0] is empty
            
        #If we set "no-cluster sizes" flag, we don't count cluster sizes
        if ncs == True:
            cs=0    # "0" instead of "1" because the read itself is already counted in "occ"
        else:
            #Account for cluster sizes when counting abundance
            #E.g.: 159758_t11_w76_x5_1_113_-_xcs22  -> "_xcs22" -> 22 similar/homologous reads
            cs=occField.split("_")[7]    #E.g.: 254208_t11_w76_x5_1_76_+_xcs22 , the field num 7 (from 0) contains the cluster size of the read
            cs=int(cs.split("xcs")[1])-1 #Create an array with 2fields, the 2 parts of the split; in this case occ.split("x")[0] is empty
                                         #We substract 1, because the read itself is already counted in "occ"

        ##Find out the ontology_id associated to the read-hit
        listeggontoids=[]
        if d1cog.has_key(str(md5id)):   #E.g. of an entry to this dictionary(key=md5,value=ontoids): c914fc2d627ac751e755acd04775d9f5 -> 35147;3245;23465
            listeggontoids=d1cog[md5id].split(';')

        if d1nog.has_key(str(md5id)):   #E.g. of an entry to this dictionary(key=md5,value=ontoids): c914fc2d627ac751e755acd04775d9f5 -> 85147;8245;83465
            listeggontoids=listeggontoids+d1nog[md5id].split(';')
        
        if not d1cog.has_key(str(md5id)) and not d1nog.has_key(str(md5id)):
            #If the read-md5 id is not in "md5_ontology_map" (COG/NOG) do not exist annotation.
            unknown+=1
            continue

        if nfc == True:   #Get only the 1st orthid if "nfc" option is selected
            listeggontoids=listeggontoids[:1] 

        #Fill "d2"(Orthid -> Abundance) with abundance per orthid.
        for eggontoid in listeggontoids:
                if d2.has_key(str(eggontoid)):
                    d2[eggontoid]+=(occ+cs)
                else:
                    d2[eggontoid]=(occ+cs)


    print(str(unknown)+"\tunknown") #Print unknowns

    #"d2" to List of Tuples
    d2LoT=sorted(d2.items(), key=lambda x:x[1],reverse=True)

    for i in d2LoT:
        #i[0] : key ; i[1] : value (int)
        print(str(i[1])+"\t"+i[0])
    
    archivo.close()

def usage():
    print(__doc__)
    stdout.flush()


if __name__ == '__main__':
    dictinit()


