#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#########################################################################
#  Metatrans - metatranscriptomic sequence analysis pipeline for the    #
#  human gut microbiota.                                                #
#                                                                       #
#  Copyright (C) 2015 Xavier Martínez VHIR - Vall d'Hebron              #
#                                            Institut de Recerca        #
#                                                                       #
#  This file is part of Metatrans.                                      #
#                                                                       #
#  Metatrans is free software: you can redistribute it and/or modify    #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  Metatrans is distributed in the hope that it will be useful,         #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with Metatrans.  If not, see <http://www.gnu.org/licenses/>.   #
#                                                                       #
#  Contact e-mail:                                                      #
#  metatrans-forum@googlegroups.com                                     #
#########################################################################



"""
Description:
    Annotates all Metahit genename ids from a SOAP2 output file,
    to EggNOG orthologous ids.

    Input,e.g:
         1323_t14_w76_x164_1_111_-  V1.UC2-0_GL0094335  70.97   31  9   0   1   31  2   32  0.001   39.3
         2756_t11_w76_x106_1_123_+  BGI-17A_GL0060198   96.77   31  1   0   11  41  1   31  2e-12   62.0

    Output,e.g:
         79465  unknown
         8075   NOG12793
         2506   COG1653
         2473   COG0574
         2267   COG0642
         2146   NOG298297

    Usage:  
        SOAP_mh14ToOrthids.py { --help | [-nr|-ncs|-nfc] --dicin1=<filename> --fin=<filename> }

        or, in short form,

        SOAP_mh14ToOrthids.py { -h | [-nr|-ncs|-nfc] -d1 <filename> -i <filename> }

        E.g:
            SOAP_mh14ToOrthids.py -d1 ./0-Databases/annotations_db/IGC.annotation.summary.v2 -i ./3-MAP/m3output/soap2/C1JB3ACXX_1_7_m3.aln.merged.mh2014.igc.soap > out.txt
"""

from __future__ import print_function
from sys import stderr


def mesgstderr(*objs):
    print("", *objs, file=stderr)


## Define Imports
import warnings
warnings.filterwarnings('ignore', '.*the popen2 module is deprecated.*', DeprecationWarning)


import time
from time import ctime #to define ctime()
start_time = time.time()

from sys      import argv, exit, stdout


## Define functions
def dictinit():

    nr=False
    ncs=False
    nfc=False


    argidx = 1    # skip arg 0
    while argidx < len(argv):
        if   argv[argidx] == '-h' or argv[argidx] == '--help':
            usage()
            exit(-1)
        elif argv[argidx] == '-nr':   #Don't count "collapsing" data from headers (ex: "..x45") 
                                      #+ we only count the 1st orthid for one read  
                                      #+ we spread out abundance according to the Multi-Letter funcat code
            nr=True
            argidx += 1
        elif argv[argidx] == '-ncs':  #Don't count "cluster size" data from headers (e.g.: "..xcs23")
            ncs=True
            argidx += 1
        elif argv[argidx] == '-nfc':  #No function collision (functions without collision): We only count the 1st orthid for one read   
            nfc=True
            argidx += 1
        elif argv[argidx] == '-d1':    
            dic1 = argv[argidx+1]
            argidx += 2
        elif argv[argidx].startswith('--dicin1'):
            splitArg = argv[argidx].split('=')
            try:
                dic1 = splitArg[1]
            except:
                usage()
                print('\n #Error# Invalid argument:', argv[argidx],' \n')
                exit(-1)
            argidx += 1
        elif argv[argidx] == '-i': 
            fin = argv[argidx+1]
            argidx += 2
        elif argv[argidx].startswith('--fin'):
            splitArg = argv[argidx].split('=')
            try:
                fin = splitArg[1]
            except:
                usage()
                print('\n #Error# Invalid argument:', argv[argidx],' \n')
                exit(-1)
            argidx += 1
        else:
            usage()
            print('\n #Error# Unrecognized argument:', argv[argidx],' \n')
            exit(-1)


    fileDic = dic1
    #fileDic = './0-Databases/metahit-2014/IGC.annotation.summary.v2'

    ## Load dictionaries to memory for fast access
    d1 = {}     #Dictionary of Metahit genename id to annotation of genenames 
    with open(fileDic) as f:
        for line in f:
            (key,val) = (line.split("\t")[1],line.strip())
            d1[key]=val

    mesgstderr( "#Finish loading dictionaries. It lasted --- %s seconds ---." % (time.time() - start_time) )


    ## Parse input file:
    archivo = open(fin) #Read soap2 output. It returns an iterator which reads the file on demand, does  not put the whole file into memory
                #E.g.:  
                #248374_t12_w76_x3_1_82_+_xcs11 TCAGGAGCAGCAGGAATCATTCGTCCAGGAAAAGGTAACATTCAGGTTATCATTGGACCAAAAGTTCAGTTTGTTGTAGAT   hhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhh   2   a   81  +   MH0062_GL0065318    1363    1   C->76T40    81M 76C4
                #249773_t13_w76_x3_1_123_-_xcs1 CAGAAGGAATGTGGACTTCCTCAGGATGAGAATCAGTTTGTAATAGGAATCATATCCAGACTCACAGACCAGAAGGGTCTTGATCTGGTTGACAGGATCATGAACGATATATGCTGTGAC    hhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhh    1   a   120 -   MH0007_GL0024521    874 0   120M    120


    d2={} #Dictionary of orthids and their abundance
          #E.g.: (key,value)="NOG342",73
          #      (key,value)="COG729",765


    #Begin parsing the file
    for line in archivo:  

        #Parse line (read-hit)
        line = line.strip('\n')  
        if not line: break    
        #print line #debug
        
        genename=line.split("\t")[7] #Index starts at 0. This returns the genename($8) of the soap2 hit
        occField=line.split("\t")[0] #Num of occurrences field (generated at collapsing step)
                
        #If we set "non-redundant" flag, we don't count collapsed copies (artificial duplicate reads) of a read.
        if nr == True:
            occ=1
        else:
            occ=occField.split("_")[3] #E.g.: 254208_t11_w76_x5_1_76_+ , the field num 3 (from 0) contains the counting of occ.
            occ=int(occ.split("x")[1]) #Create an array with 2fields, the 2 parts of the split; in this case occ.split("x")[0] is empty
            
        #If we set "no-cluster sizes" flag, we don't count cluster sizes
        if ncs == True:
            cs=0    # "0" instead of "1" because the read itself is already counted in "occ"
        else:
            #Account for cluster sizes when counting abundance
            #E.g.: 159758_t11_w76_x5_1_113_-_xcs22  -> "_xcs22" -> 22 similar/homologous reads
            cs=occField.split("_")[7]    #E.g.: 254208_t11_w76_x5_1_76_+_xcs22 , the field num 7 (from 0) contains the cluster size of the read
            cs=int(cs.split("xcs")[1])-1 #Create an array with 2fields, the 2 parts of the split; in this case occ.split("x")[0] is empty
                                         #We substract 1, because the read itself is already counted in "occ"

        #Find out the annotation associated to the read-hit
        if d1.has_key(genename):    #E.g. of an entry to this dictionary: 79929.MTBMA_c00390
            annot=d1[genename]  #E.g.:
                                #[key]MH0293_GL0136119
                                #[value]47      MH0293_GL0136119        22731   Lack 3'-end     EUR     unknown unknown K01824  COG5184 0.0165745856353591      0.0177570093457944      Lipid Metabolism        Cell cycle control, cell division, chromosome partitioning;Cytoskeleton EUR
        else:
            usage()
            print('\n #Error# Genename "'+genename+'" is not annotated. This should NEVER happen, all Metahit genes are annotated. \n')
            exit(-1)

        #[8]  -> EggNOGv3 Ids
        listeggids=[]
        eggids=annot.strip().split("\t")[8]
        listeggids=eggids.split(';')   #We get directly the orthid (EggNOGv3) from the Metahit annotation.

        if nfc == True:   #Get only the 1st orthid
            listeggids=listeggids[:1] 

        #Fill "d2"(Orthid -> Abundance) with abundance per orthid.
        for eggid in listeggids:
            if d2.has_key(eggid):
                d2[eggid]+=(occ+cs)
            else:
                d2[eggid]=(occ+cs)

    #"d2" to List of Tuples
    d2LoT=sorted(d2.items(), key=lambda x:x[1],reverse=True)

    for i in d2LoT:
        #i[0] : key ; i[1] : value (int)
        print(str(i[1])+"\t"+i[0])

    archivo.close()

def usage():
    print(__doc__)
    stdout.flush()


if __name__ == '__main__':
    dictinit()


