#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#########################################################################
#  Metatrans - metatranscriptomic sequence analysis pipeline for the    #
#  human gut microbiota.                                                #
#                                                                       #
#  Copyright (C) 2015 Xavier Martínez VHIR - Vall d'Hebron              #
#                                            Institut de Recerca        #
#                                                                       #
#  This file is part of Metatrans.                                      #
#                                                                       #
#  Metatrans is free software: you can redistribute it and/or modify    #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  Metatrans is distributed in the hope that it will be useful,         #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with Metatrans.  If not, see <http://www.gnu.org/licenses/>.   #
#                                                                       #
#  Contact e-mail:                                                      #
#  metatrans-forum@googlegroups.com                                     #
#########################################################################


"""
    Description:
        Reads from a table within a file and get data from it.

    Input,e.g:
        #SampleName SampleFiles Shortname   Description Order   Type
        C1HVFACXX_8_7_1000  C1HVFACXX_8_7_1000.1.fastq,C1HVFACXX_8_7_1000.2.fastq   P1-BFm  P21.V3m 1   mrna
        C1JB3ACXX_1_5_1000  C1JB3ACXX_1_5_1000.1.fastq,C1JB3ACXX_1_5_1000.2.fastq   C1-BF   S08.V3  2   totalrna
        
        getMeta.py -posbycolname SampleName,1 -i ./0-Software/config/metadata.txt
        getMeta.py -pos 1,6 -i ./0-Software/config/metadata.txt 
        getMeta.py -pos 1,6 -byorder -i ./0-Software/config/metadata.txt 
        getMeta.py -colheaders -i ./0-Software/config/metadata.txt 
        
    Output,e.g. One line outupt per call:
        C1HVFACXX_8_7_1000
        6
        3
        ('SampleName', 'SampleFiles', 'Shortname', 'Description', 'Order', 'Type')

    Usage:  
        getMeta.py [-h] {-ncols | -nrows [-byorder] | -colheaders | -pos <POSITION> [-byorder] | -posbycolname <POSBYCOLNAME> [-byorder]} -i <INPUT_FILE>
"""


#Argparse
import argparse #Added in Python 2.7 (we are using Python >=2.7.3) 
import numpy as np
import os.path
import sys
import textwrap as tw
 

#Customize some argparse messages:
lookup = {
"usage: ": "Usage: ",
"optional arguments": "Optional arguments"}
def gettext(s):
    return lookup.get(s, s)
argparse._ = gettext


def getOrderRow(mydata,row):
    aorder=mydata['Order']
    asort=np.zeros(shape=len(aorder),dtype=[('nrow',int),('Order',int)])
    for i in range(0,len(aorder)):
        asort[i]=(i+1,aorder[i])
    asort.sort(order='Order')
    disabled=len(asort[np.where( asort['Order'] == 0 )])  #Delete all entries where sample is not going to be processed
    asort=np.delete(asort,[range(0,disabled)])
    #Re-assign 'Order' sequentially starting at 1. Eg.:  a=np.array([(3, 2),(2, 4),(6, 7)]) -> array([(3, 1),(2, 2),(6, 3)]) (asort[0][0] -> 'nrow'/asort[0][1] -> 'Order')
    for i in range(0,len(asort)):
        asort[i][1]=i+1
    return int(asort[np.where( asort['Order'] == row )][0]['nrow'])

def getOrderRowSize(mydata):
    aorder=mydata['Order']
    asort=np.zeros(shape=len(aorder),dtype=[('nrow',int),('Order',int)])
    for i in range(0,len(aorder)):
        asort[i]=(i+1,aorder[i])
    asort.sort(order='Order')
    disabled=len(asort[np.where( asort['Order'] == 0 )])  #Delete all entries where sample is not going to be processed
    asort=np.delete(asort,[range(0,disabled)])
    return len(asort['Order'])

#Add two format_classes:
#formatter_class="argparse.RawTextHelpFormatter" to disable autoformatting help text. 
#"argparse.ArgumentDefaultsHelpFormatter" to add default values.
#Use "default=argparse.SUPPRESS" to hide any default value when "argparse.ArgumentDefaultsHelpFormatter" is enabled.
class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter, argparse.RawTextHelpFormatter):
    pass
    

#Add optional dependency for an optional argument:
class UserAction(argparse.Action):
    def __init__(self, option_strings, dest, default=None, required=False, help=None):
        super(UserAction, self).__init__(option_strings, dest, nargs=0, const=None, 
                                        default=False, required=required, help=help) # We set here the default value for "byorder"
    def __call__(self, parser, namespace, values, option_string=None):
        if not hasattr(namespace, 'pos') and not hasattr(namespace, 'posbycolname') and not hasattr(namespace, 'nrows'):   #Check if list is empty
            parser.error('option -nrows/-pos/-posbycolname is required with -byorder option')
        else:
            setattr(namespace, self.dest, True)
    
 

if __name__ == "__main__": 
    
    parser = argparse.ArgumentParser(usage='%(prog)s [-h] {-ncols | -nrows [-byorder] | -colheaders | -pos <POSITION>  [-byorder] | -posbycolname <POSBYCOLNAME>  [-byorder]} -i <INPUT_FILE>',
                                     description='Gets data from a table stored in a file.\n\n'+
                                     tw.dedent('''\
    [] indicates optional input (order unimportant)
    {} indicates required input (order unimportant)
                                     '''),
                                     add_help=False,
                                     formatter_class=CustomFormatter) 

    parser.add_argument("-h", "--help", action="help", help="Show this help message and exit")
    parser.add_argument('-byorder', dest='byorder', action=UserAction, default=argparse.SUPPRESS, help="Follow row-order as interpreted from 'Order' column (0-non processed row).")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('-ncols', dest='ncols', action='store_true', default=argparse.SUPPRESS, help='Returns number of columns') #Boolean. Optional argument. Stores "True" in case "-ncols" appear in variable "ncols".
    group.add_argument('-nrows', dest='nrows', action='store_true', default=argparse.SUPPRESS, help='Returns number of rows') #Boolean. Optional argument. Stores "True" in case "-nrows" appear in variable "nrows".
    group.add_argument('-colheaders', dest='colheaders', action='store_true', default=argparse.SUPPRESS, help='Returns header names (i.e. column names)') #Boolean. Optional argument. Stores "True" in case "-colheaders" appear in variable "colheaders".
    group.add_argument('-pos',  dest='pos', metavar='<POSITION>', nargs=1, action="store", default=argparse.SUPPRESS, help='\n'.join(tw.wrap('Row and column, separated by a comma, from where you want to get the value in the table. Rows and columns start at 1. Eg.: 5,3 ',width=55,replace_whitespace=False,break_long_words=False))) #String.Optional argument
    group.add_argument('-posbycolname',  dest='posbycolname', metavar='<POSBYCOLNAME>', nargs=1, action="store", default=argparse.SUPPRESS, help='\n'.join(tw.wrap('ColumnNname and row, separated by a comma, from where you want to get the value in the table. Rows start at 1. Eg.: SampleName,3 ',width=55,replace_whitespace=False,break_long_words=False)) ) #String.Optional argument
    parser.add_argument('-i',  dest='i', metavar='<INPUT_FILE>', nargs=1, action="store", default=argparse.SUPPRESS, help='\n'.join(tw.wrap('\
Input formatted \ntabular file. The file must contain at least three columns with: "#SampleName"(\'#\' is omitted), two "SampleFiles" separated by a comma, \
"Order" with a sequential number beginning with 1 to specify the order in which the samples will be processed (if 0 the sample won\'t be processed), "Shortname" with a short sample\
name that will be used to display graphics, and at least one \
factor "F<number>-<factor_name>" to perform differential expression analysis. Other columns can be added to fully describe the experiment but\
won\'t be parsed.',width=55,replace_whitespace=True,break_long_words=False))+\
    '\n\nE.g. required columns in a metadata file(delimiter="\\t"): \n\
    #SampleName         SampleFiles                                             Shortname   Order   F1-Diet\n\
    C1HVFACXX_8_7_1000  C1HVFACXX_8_7_1000.1.fastq,C1HVFACXX_8_7_1000.2.fastq   P1-BFm      1       BF     \n\
    C1JB3ACXX_1_5_1000  C1JB3ACXX_1_5_1000.1.fastq,C1JB3ACXX_1_5_1000.2.fastq   C1-AF       2       AF     \n\
    C1JB3ACXX_1_6_1000  C1JB3ACXX_1_6_1000.1.fastq,C1JB3ACXX_1_6_1000.2.fastq   C1-AF       3       AF     \n'
    '\nE.g. full detailed metadata file(delimiter="\\t"): \n\
    #SampleName         SampleFiles                                             Shortname Description                         Order   Type      F1-Paired   F2-Diet \n\
    C1HVFACXX_8_7_1000  C1HVFACXX_8_7_1000.1.fastq,C1HVFACXX_8_7_1000.2.fastq   P1-BFm    P21.V3m-individuals from Europe     1       mrna      I1          BF \n\
    C1JB3ACXX_1_5_1000  C1JB3ACXX_1_5_1000.1.fastq,C1JB3ACXX_1_5_1000.2.fastq   C1-AF     S08.V4-individuals from Asia        2       totalrna  I1          AF \n\
    C1JB3ACXX_1_6_1000  C1JB3ACXX_1_6_1000.1.fastq,C1JB3ACXX_1_6_1000.2.fastq   C2-AF     S09.V4-individuals from Asia        3       totalrna  I2          AF \n'
    , required=True) #String.Optional argument forced to Required.

    #print parser.parse_args('-h'.split()) #debug
    #args = parser.parse_args(('-colheaders -i file').split()) #debug
    #args = parser.parse_args(("-posbycolname 'SampleName',3 -i file").split()) #debug
    #print args #debug

    args = parser.parse_args()
    
    #Set internal default values
    if not hasattr(args, 'byorder'): args.byorder=False
    if not hasattr(args, 'ncols'): args.ncols=False #We supress its default value from help because it is just a flag, but we set its default value "internally".
    if not hasattr(args, 'nrows'): args.nrows=False
    if not hasattr(args, 'colheaders'): args.colheaders=False
    if not hasattr(args, 'pos'): args.pos=None  
    if not hasattr(args, 'posbycolname'): args.posbycolname=None
    
    
    
    ##Import file:
    #filename="./0-Software/config/metadata.txt" #debug
            #Which contains something similar to this:
            # #SampleName   SampleFiles Shortname   Description Order   Type    F1-Paired   F2-Diet
            # C1HVFACXX_8_7_1000    C1HVFACXX_8_7_1000.1.fastq,C1HVFACXX_8_7_1000.2.fastq   P1-BFm  P21.V3m 1   mrna    I1  BF
            # C1JB3ACXX_1_5_1000    C1JB3ACXX_1_5_1000.1.fastq,C1JB3ACXX_1_5_1000.2.fastq   C1-BF   S08.V3  2   totalrna    I2  BF

            
    d=','   #Delimiter char to specify table positions.
    filename="".join(args.i)    #Customize this default value, otherwise this is printed automatically "(default: <open file '<stdin>', mode 'r' at 0x7f5c109160c0>)"   
    if not os.path.isfile(filename):      #If file exists
        sys.exit('\n #Error# File <'+''.join(args.i)+'> does not exist. \n') 
    else:
        mydata = np.genfromtxt(filename,names=True,dtype=None,delimiter='\t')   #Get table-data. Note: "#SampleName" is taken as "SampleName" without "#".
        #delimiter (delimiter=None): str, int, or sequence, optional:
        #The string used to separate values. By default, any consecutive whitespaces act as delimiter. 
        #An integer or sequence of integers can also be provided as width(s) of each field.

        if args.ncols:
            print len(mydata[0])
        elif args.nrows:
            if args.byorder: nrows=getOrderRowSize(mydata)
            else: nrows=len([row[0] for row in mydata])
            print nrows
        elif args.colheaders:
            print mydata.dtype.names
        elif args.pos is not None: 
            row=int(''.join(args.pos).split(d)[0])
            col=int(''.join(args.pos).split(d)[1])
            if args.byorder: row=getOrderRow(mydata,row)
            print mydata[row-1][col-1]  #[row][col]
        elif args.posbycolname is not None:
            colname=''.join(args.posbycolname).split(d)[0]
            row=int(''.join(args.posbycolname).split(d)[1])
            if args.byorder: row=getOrderRow(mydata,row)
            print mydata[colname][row-1]        #[colname][row]











