#!/usr/bin/awk -f

#########################################################################
#  Metatrans - metatranscriptomic sequence analysis pipeline for the    #
#  human gut microbiota.                                                #
#                                                                       #
#  Copyright (C) 2015 Xavier Martínez VHIR - Vall d'Hebron              #
#                                            Institut de Recerca        #
#                                                                       #
#  This file is part of Metatrans.                                      #
#                                                                       #
#  Metatrans is free software: you can redistribute it and/or modify    #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  Metatrans is distributed in the hope that it will be useful,         #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with Metatrans.  If not, see <http://www.gnu.org/licenses/>.   #
#                                                                       #
#  Contact e-mail:                                                      #
#  metatrans-forum@googlegroups.com                                     #
#########################################################################


#Usage:
#   mh_abundance_totable.awk  -v shns="C1-BF\tC2-BF\tC1-AF..."  file1 file2 file3 ... > table.tsv

#Description:
#   Summarize all metahit hits from all samples to table format.
#   Singletons (elements that only exist in 1 sample) are removed.

#Awk version: GNU Awk 3.1.8 (gawk)


BEGIN { 
    split(shns,headers,"\t")
}   
{
    if(FNR == 1) {
        f++;
    }

    #Entry example:
    #$1 $2  $3
    #9  [G] Carbohydrate transport and metabolism
    #8  [S] Function unknown
    
    #We create a "matrix". 1dimension for samples. 2ndimension for nº of elems per each function
    #If the element appears for the 1st time we count it.
    
    id=getfieldtillend(2)
    
    #Multi associative array has number labels (f variable): "1,2,.." to refer to samplenames
    items[id,f]+=$1;            #Here we count the number of reads annotated to the same function + the number of different reads 
    
} 
END {
    printdata("ID",headers,items);
    system("rm -f " tfile);     #Delete temporal file  (this file is created in printdata)
}


function getfieldtillend(nfield) 
{
    out="";
    for (i = nfield; i <= NF; i++) { 
          out = out $i " ";
    }                             
    return out;
}

function printdata(tabname,headers,funcdic)
{
        
        #Temporal file used for sorting
        tfile="tmp.dat";
        split("", visited); #Clean visited dicctionary for the next printing
                
        #Print Headers
        printf("%s",tabname) > tfile;
        for (h = 1; h <= f; h++) {
            printf("\t%s", headers[h]) > tfile;
        }
        printf("\n") > tfile;   
        
        #Print Data
        #i coordinate: name of each element  = label[1] .Eg: "Function unknown","Carbohydrate transport and metabolism",...
        #j coordinate: name of each sample   = label[2] .Eg: 1,2,3,...
        for (ij in funcdic) {
            split(ij,label,SUBSEP);
            
            if(!(label[1] in visited)) visited[label[1]]="VISITED"; 
            else continue;
            
            #SINGLETON REMOVAL
            cs=0; #singleton counter
            ps=0; #position of the sample with the singleton
            for (k = 1; k <= f; k++) {
                if( ((label[1],k) in funcdic)) {    #Check if the element exist for that sample
                    cs++;
                    ps=k;
                }
            }
            if(cs == 1 && funcdic[label[1],ps] == 1) { continue; }  #Jump to the next element if it is a singleton. 
            
            item=label[1];
            sub(/ *$/, "", item);   #Remove any trailing whitespace
            printf("\"%s\"",item) > tfile;  #Print the name of the element, we put double quotes for compatibility with R.
            
            av=0;  #Calc average per row.
            for (k = 1; k <= f; k++) {  #Print abundance per each sample.
                if(!((label[1],k) in funcdic)) printf("\t%s","0") > tfile;  #If the element for the sample doesn't exist print 0
                else {
                    printf("\t%s",funcdic[label[1],k]) > tfile;
                    av+=funcdic[label[1],k];
                }
            }
            printf("\t%f",(av/f)) > tfile;
            printf("\n") > tfile;
        }

        #Sort table:
        system("(head -n+1 " tfile ";cat " tfile "|tail -n+2|sort -nr -k " (f+2) "," (f+2) " -t '	' | awk -vN=1 'BEGIN{FS=OFS=\"\t\"}{if(NF<N){NF=0}else{NF-=N}print}')");    #This prints output sorted
        
        system("cat /dev/null > " tfile);   #Empty the file
        close(tfile);  #Redirection must be closed to process the same file again
}

